﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ELTE.Forms.Switchboard
{
    public partial class MainForm : Form
    {
        private DateTime _StartTime; // a program indulási ideje
        private List<Call> _CallList; // a hívásokat eltároljuk egy listában
        private Timer _CallTimer; // időzítő
        private Button[] _ExtensionButtons; // a mellékek gombjai egy tömbben
        private Int32[] _CallTimes; // a mellékekhez tartozó aktuális hívásidők
        private Int32 _TimeToNextCall; // a következő hívásig hátramaradt idő (másodpercben)
        private Int32 _NumberOfGoodConnections; // a jó kapcsolások száma
        private Int32 _NumberOfBadConnections; // a hibás kapcsolások száma

        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e) // a betöltést követő eseményhez (Load) rendelt eseménykezelő
        {
            SettingsForm extensionForm = new SettingsForm(); // mellékek számát külön dialógusban kérjük be
            extensionForm.ShowDialog(); // ez a dialógusablak betöltődik még a főablak előtt

            _StartTime = DateTime.Now;

            // létrehozzuk a tömböket és a listát:
            _CallList = new List<Call>();
            _CallTimes = new Int32[extensionForm.ExtensionCount];
            _ExtensionButtons = new Button[extensionForm.ExtensionCount];

            // inicializáljuk a gombokat:
            for (Int32 i = 0; i < extensionForm.ExtensionCount; i++)
            {
                _ExtensionButtons[i] = new Button();
                _ExtensionButtons[i].Location = new System.Drawing.Point(150 + 30 * (i % 10), 10 + 30 * (i / 10));
                _ExtensionButtons[i].Name = i.ToString();
                _ExtensionButtons[i].Size = new System.Drawing.Size(30, 30);
                _ExtensionButtons[i].Text = (i + 1).ToString();
                _ExtensionButtons[i].BackColor = Color.White;
                _ExtensionButtons[i].Click += new System.EventHandler(ExtensionButton_Click);

                Controls.Add(_ExtensionButtons[i]); // hozzáadjuk a form vezérlőihez
            }

            Random random = new Random(DateTime.Now.Millisecond);
            _TimeToNextCall = random.Next(5, 10); // beállítjuk a következő hívás idejét

            _CallTimer = new Timer(); // beállítjuk az időzítőt
            _CallTimer.Interval = 1000; // egy másodpercenként lesz időzített esemény 
            _CallTimer.Tick += new EventHandler(CallTimer_Tick);
            _CallTimer.Start(); // elindítjuk az időzítőt
            _ListBoxCalls.Items.Add("Telefonközpont indul");

        }

        private void CallTimer_Tick(object sender, EventArgs e)
        {
            // először csökkentjük a gomboknál megmaradt hívásidőt
            for (Int32 i = 0; i < _CallTimes.Length; i++)
                if (_CallTimes[i] > 0) // ha maradt még hívásidő
                {
                    _CallTimes[i]--; // akkor csökken
                    if (_CallTimes[i] == 0) // ha teljesen elfogyott
                    {
                        _ExtensionButtons[i].Enabled = true; // akkor a gomb újra normális állapotba kerül
                        _ExtensionButtons[i].BackColor = Color.White;
                    }
                }
            _TimeToNextCall--;
            if (_TimeToNextCall == 0) // ha elfogyott a következő hívásig megmaradt idő
            {
                if (_CallList.Count > 0 && !_CallList[_CallList.Count - 1].Connected) // ha az előző hívást nem kapcsolta
                {
                    _NumberOfBadConnections++; // számolunk egy rossz kapcsolást
                    _LabelBadCount.Text = _NumberOfBadConnections.ToString();
                }

                Call call = new Call(_ExtensionButtons.Length); // új hívás
                _CallList.Add(call); // felvesszük a hívások közé
                _ListBoxCalls.Items.Insert(0, _CallList.Count.ToString() + ", " + DateTime.Now.ToString("hh:mm:ss") + ", " + call.Length.ToString() + ", " + call.Extension.ToString()); // kiíratjuk a listába
                Random random = new Random(DateTime.Now.Millisecond);

                _TimeToNextCall = random.Next(5, 10); // beállítjuk a következő hívás idejét

                if (_CallTimer.Interval > 100)
                    _CallTimer.Interval = Convert.ToInt32(_CallTimer.Interval * 0.95); // gyorsítjuk az órát
            }
        }

        private void ExtensionButton_Click(object sender, EventArgs e)
        {
            if (_CallList.Count > 0 && !_CallList[_CallList.Count - 1].Connected) // ha van hívás, amit még nem kapcsoltak
            {
                if (_CallList[_CallList.Count - 1].Extension == Int32.Parse((sender as Button).Text)) // a megfelelő mellékre kapcsolják
                {
                    Int32 index = Int32.Parse((sender as Button).Name);
                    _CallTimes[index] = _CallList[_CallList.Count - 1].Length; // akkor rögzítjük a hívásidőt
                    _ExtensionButtons[index].BackColor = Color.Blue; // a gombot kékre színezzük
                    _ExtensionButtons[index].Enabled = false; // és kikapcsoljuk
                    _NumberOfGoodConnections++; // növeljük a jó kapcsolások számát
                    _LabelGoodCount.Text = _NumberOfGoodConnections.ToString();
                    _CallList[_CallList.Count - 1].Connected = true; // beállítjuk a hívást kapcsoltra
                }
                else // ha rossz mellékre kapcsolt
                {
                    Int32 index = Int32.Parse((sender as Button).Name);
                    _CallTimes[index] = 1; // 1 másodpercig...
                    _ExtensionButtons[index].BackColor = Color.Red; //...pirosra színezzük a gombot, hogy látszódjon
                     _NumberOfBadConnections++; // növeljük a rossz kapcsolások számát
                    _LabelBadCount.Text = _NumberOfBadConnections.ToString();
                    _CallList[_CallList.Count - 1].Connected = true;
                }
            }
        }

        private void ButtonClose_Click(object sender, EventArgs e)
        {
            // egy szövegdobozban jelenítjük meg az eredményt:
            MessageBox.Show("A telefonközpont " + (DateTime.Now - _StartTime).ToString() + 
                            " ideig futott\nÖsszesen " + _CallList.Count + " hívást fogadott, amelyből " +
                            _NumberOfGoodConnections.ToString() + "-t jó mellékre kapcsolt.\nA jó kapcsolások aránya " + 
                            (100.0 * (_NumberOfGoodConnections) / _CallList.Count).ToString() + 
                            "%.\nA hibás kapcsolások száma " + _NumberOfBadConnections.ToString() + " volt.", 
                            "Telefonközpont vége");
            Close();
        }
    }
}
